/* 
          LTR212.
            ,    
           :
    ltr212_recv  slot crate_serial srvip
    :
        slot         -   ( 1  16)
        crate_serial -    (   )
        srvip        - ip-  ltrd  LtrServer (  )
     :    ,       

       ,    1.30.8    
       NULL    bio-     
        (Windows)     (Linux).


        4- .
             .

         ,   
       .

           Windows   CTRL+C  Linux.

      VisualStudio:
          Visual Studio,     
    ( (Project) ->  (Properties) ->   (Configuration Properties)
    -> /++ ->  (General) ->    (Additional Include Directories))
            (ltr212api.h    
         ltr/include   )
        .lib   <  >/lib/msvc
    ( (Project) ->  (Properties) ->   (Configuration Properties) ->
     (Linker) ->  (General) ->    (Additional Library Directories)).

    !:      Visual Studio    
       ,    :
      (File) ->    (Advanced Save Options)...
        (Encoding)   (UTF8,  )/Unicode (UTF-8 with signature)
        .
*/


#include "ltr/include/ltr212api.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef _WIN32
#include <locale.h>
#include <conio.h>
#else
#include <signal.h>
#include <unistd.h>
#endif



/* -   ,     */
#define RECV_CH_SIZE       256

/*      */
static int f_out = 0;

typedef struct {
    int slot;
    const char *serial;
    DWORD addr;
} t_open_param;

static const char* f_module_type_str[] = {
    "LTR212/LTR212-M3",
    "LTR212-M1",
    "LTR212-M2"
};

/*    .   ,  
 *   :
 * 1  -   ( 1  16)
 * 2  -   
 * 3  - ip-  */
static int f_get_params(int argc, char** argv, t_open_param* par) {
    int err = 0;
    par->slot = LTR_CC_CHNUM_MODULE1;
    par->serial = "";
    par->addr = LTRD_ADDR_DEFAULT;


    if (argc > 1)
        par->slot = atoi(argv[1]);
    if (argc > 2)
        par->serial = argv[2];
    if (argc > 3) {
        int a[4],i;
        if (sscanf(argv[3], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3])!=4) {
            fprintf(stderr, "  IP-!!\n");
            err = -1;
        }

        for (i=0; (i < 4) && !err; i++) {
            if ((a[i]<0) || (a[i] > 255)) {
                fprintf(stderr, " IP-!!\n");
                err = -1;
            }
        }

        if (!err) {
            par->addr = (a[0] << 24) | (a[1]<<16) | (a[2]<<8) | a[3];
        }
    }
    return err;
}



#ifndef _WIN32
/*     Linux */
static void f_abort_handler(int sig) {
    f_out = 1;
}
#endif


int main(int argc, char** argv) {
    INT err = LTR_OK;
    TLTR212 hltr212;
    t_open_param par;
#ifndef _WIN32
    struct sigaction sa;
    /*   Linux      ,
           */
    sa.sa_handler = f_abort_handler;
    sigaction(SIGTERM, &sa, NULL);
    sigaction(SIGINT, &sa, NULL);
    sigaction(SIGABRT, &sa, NULL);
#endif

#ifdef _WIN32
    /*         Windows*/
    setlocale(LC_ALL, "");
#endif

    err = f_get_params(argc, argv, &par);

    if (!err) {
        LTR212_Init(&hltr212);
        /*    .    
             DSP*/
        err = LTR212_Open(&hltr212, par.addr, LTRD_PORT_DEFAULT, par.serial, par.slot, NULL);
        if (err != LTR_OK) {
            fprintf(stderr, "     .  %d (%s)\n",
                    err, LTR212_GetErrorString(err));
        } else {
            /*     ,     */
            printf("  !\n");
            printf("       = %s\n", hltr212.ModuleInfo.Name);
            printf("        = %s\n", hltr212.ModuleInfo.Serial);
            printf("            = %s\n", hltr212.ModuleInfo.Type <
                   sizeof(f_module_type_str)/sizeof(f_module_type_str[0]) ?
                        f_module_type_str[hltr212.ModuleInfo.Type] : " ");
            printf("       = %s\n", hltr212.ModuleInfo.BiosVersion);
            printf("       = %s\n",   hltr212.ModuleInfo.BiosDate);
            fflush(stdout);

            /*    EEPROM */
            err = LTR212_TestEEPROM(&hltr212);
            if (err!=LTR_OK) {
                fprintf(stderr, "    EEPROM: %d (%s)\n",
                        err, LTR212_GetErrorString(err));
            }

            if (err==LTR_OK) {
                INT ch;
                /* -------------    --------------- */

                /*   3-   */
                hltr212.AcqMode = LTR212_FOUR_CHANNELS_WITH_HIGH_RESOLUTION;
                /*     */
                hltr212.UseClb = FALSE;
                 /*    */
                hltr212.UseFabricClb = TRUE;
                /*   4-  */
                hltr212.LChQnt = 4;
                /*    5  */
                hltr212.REF = LTR212_REF_5V;
                /*    */
                hltr212.AC = FALSE;

                /*    (     +/-80 ) */
                for (ch=0; ch < hltr212.LChQnt; ch++) {
                    hltr212.LChTbl[ch]=LTR212_CreateLChannel(ch+1, LTR212_SCALE_B_80);
                }

                err = LTR212_SetADC(&hltr212);
                if (err!=LTR_OK) {
                    fprintf(stderr, "    :  %d (%s)\n",
                            err, LTR212_GetErrorString(err));
                }
            }

            if (err==LTR_OK) {
                DWORD *wrds=NULL;
                double *data=NULL;
                INT recv_data_cnt = hltr212.LChQnt * RECV_CH_SIZE;
                //    2- 
                INT recv_wrds_cnt = 2*recv_data_cnt;
                DWORD tout;
                INT Run = 0;
                INT frame_num = 0;

                tout = LTR212_CalcTimeOut(&hltr212, RECV_CH_SIZE);

                /*        */
                wrds = (DWORD*)malloc(recv_wrds_cnt*sizeof(wrds[0]));
                data = (double*)malloc(recv_data_cnt*sizeof(data[0]));

                if ((wrds==NULL) || (data==NULL)) {
                    fprintf(stderr, "  !\n");
                    err = LTR_ERROR_MEMORY_ALLOC;
                }

                if (err==LTR_OK) {
                    err = LTR212_Start(&hltr212);
                    if (err!=LTR_OK) {
                        fprintf(stderr, "    !  %d (%s)\n",
                                err, LTR212_GetErrorString(err));
                    } else {
                        printf("  .    %s\n",
#ifdef _WIN32
                               " "
#else
                               "CTRL+C"
#endif
                               );
                        fflush(stdout);
                        Run = 1;
                    }                    
                }

                while (!f_out && (err==LTR_OK)) {
                    INT recv_cnt=0;
                    /*      */
                    recv_cnt = LTR212_Recv(&hltr212, wrds, NULL, recv_wrds_cnt, tout);
                    /*       */
                    if (recv_cnt<0) {
                        err = recv_cnt;
                        fprintf(stderr, "   :  %d (%s)\n",
                                err, LTR212_GetErrorString(err));
                    }
                    else if (recv_cnt < recv_wrds_cnt) {
                        fprintf(stderr, "  ,    !  %d,  %d\n",
                                recv_wrds_cnt, recv_cnt);
                        err = LTR_ERROR_RECV_INSUFFICIENT_DATA;
                    } else {

                        /*     */
                        err = LTR212_ProcessData(&hltr212, wrds, data, (DWORD*)&recv_cnt, TRUE);
                        if (err!=LTR_OK) {
                            printf("  !  %d (%s)\n",
                                   err, LTR212_GetErrorString(err));
                        } else {
                            int i;
                            frame_num++;
                            /*      ,
                             *      */
                            printf("   %d:  : ", frame_num);
                            for (i=0; (i < hltr212.LChQnt) && (i < recv_cnt); i++)
                                printf(" %.2f ", data[i]*1000);
                            printf("\n");
                            fflush(stdout);
                        }

                    }

#ifdef _WIN32
                    /*      */
                    if (err==LTR_OK) {
                        if (_kbhit())
                            f_out = 1;
                    }
#endif
                } // while(!f_out && (err==LTR_OK))

                /*    ,     */
                if (Run) {
                    INT stoperr = LTR212_Stop(&hltr212);
                    if (stoperr==LTR_OK) {
                        printf("  !\n");
                    } else {
                        fprintf(stderr, "   .  %d (%s)\n",
                                err, LTR212_GetErrorString(err));
                        if (err==LTR_OK)
                            err = stoperr;
                    }
                }

                free(wrds);
                free(data);
            }
        }
        LTR212_Close(&hltr212);
    }


    return err;
}
